/**
 * @file LiveServerMediaSubsession.cpp
 * @author 吴晨
 * @brief 适配 live555
 * 
 * @copyright Copyright (c) 2022-2024 OurEDA
 */

#include "LiveServerMediaSubsession.hh"

#include "liveMedia.hh"

#include "BaseDeviceSource.hh"
#include "H26XDeviceSource.hh"
#include "MJPEGDeviceSource.hh"

LiveServerMediaSubsession *LiveServerMediaSubsession::createNew(UsageEnvironment &env, codec_type_t codecType) {
    return new LiveServerMediaSubsession(env, codecType);
}

LiveServerMediaSubsession::LiveServerMediaSubsession(UsageEnvironment &env, codec_type_t codecType)
    : OnDemandServerMediaSubsession(env, True), fCodecType(codecType), fDeviceSource(NULL) {}

LiveServerMediaSubsession::~LiveServerMediaSubsession() {}

FramedSource *LiveServerMediaSubsession::createNewStreamSource(unsigned clientSessionId, unsigned &estBitrate) {
    if (fCodecType == ROV_H264) {
        estBitrate = 3000; // kbps, estimate
        H26XDeviceSource *streamSource = H26XDeviceSource::createNew(envir(), False);
        fDeviceSource = streamSource;
        return H264VideoStreamDiscreteFramer::createNew(envir(), streamSource, True);
    } else if (fCodecType == ROV_H265) {
        estBitrate = 2500;
        H26XDeviceSource *streamSource = H26XDeviceSource::createNew(envir(), True);
        fDeviceSource = streamSource;
        return H265VideoStreamDiscreteFramer::createNew(envir(), streamSource, True);
    } else if (fCodecType == ROV_JPEG) {
        estBitrate = 8000;
        MJPEGDeviceSource *streamSource = MJPEGDeviceSource::createNew(envir());
        fDeviceSource = streamSource;
        return streamSource;
    } else if (fCodecType == ROV_AAC) {
        estBitrate = 64;
        return NULL;
    } else if (fCodecType == ROV_G711A || fCodecType == ROV_G711U) {
        estBitrate = 128;
        return NULL;
    }
    return NULL;
}

RTPSink *LiveServerMediaSubsession::createNewRTPSink(Groupsock *rtpGroupsock, unsigned char rtpPayloadTypeIfDynamic, FramedSource *inputSource) {
    if (fCodecType == ROV_H264) {
        OutPacketBuffer::increaseMaxSizeTo(512 * 1024);
        return H264VideoRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic);
    } else if (fCodecType == ROV_H265) {
        OutPacketBuffer::increaseMaxSizeTo(512 * 1024);
        return H265VideoRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic);
    } else if (fCodecType == ROV_JPEG) {
        OutPacketBuffer::increaseMaxSizeTo(2 * 512 * 1024);
        return JPEGVideoRTPSink::createNew(envir(), rtpGroupsock);
    } else if (fCodecType == ROV_AAC) {
        return SimpleRTPSink::createNew(envir(), rtpGroupsock, 0, 8000, "audio", "AAC", 1, False);
    } else if (fCodecType == ROV_G711A) {
        return SimpleRTPSink::createNew(envir(), rtpGroupsock, 0, 8000, "audio", "PCMA", 1, False);
    } else if (fCodecType == ROV_G711U) {
        return SimpleRTPSink::createNew(envir(), rtpGroupsock, 0, 8000, "audio", "PCMU", 1, False);
    }
    return NULL;
}

void LiveServerMediaSubsession::closeStreamSource(FramedSource *inputSource) {
    fDeviceSource = NULL;
    OnDemandServerMediaSubsession::closeStreamSource(inputSource);
}
