/**
 * @file MJPEGDeviceSource.hh
 * @author 吴晨
 * @brief 适配 live555
 *
 * @copyright Copyright (c) 2022-2024 OurEDA
 */

#ifndef _MJPEG_DEVICE_SOURCE_HH
#define _MJPEG_DEVICE_SOURCE_HH

#include "JPEGVideoSource.hh"
#include "readerwritercircularbuffer.h"

#include "BaseDeviceSource.hh"
#include "JpegFrameParser.hh"

class MJPEGDeviceSource : public JPEGVideoSource, public BaseDeviceSource {
public:
    static MJPEGDeviceSource *createNew(UsageEnvironment &env);

public:
    virtual u_int8_t type() { return parser.type(); }
    virtual u_int8_t qFactor() { return parser.qFactor(); }
    virtual u_int8_t width() { return parser.width(); }
    virtual u_int8_t height() { return parser.height(); }
    virtual u_int8_t const *quantizationTables(u_int8_t &precision, u_int16_t &length) {
        precision = parser.precision();
        return parser.quantizationTables(length);
    }
    virtual u_int16_t restartInterval() { return parser.restartInterval(); }
    virtual codec_type_t getCodecType();
    virtual void signalNewFrame(const FrameData &frameData);

protected:
    MJPEGDeviceSource(UsageEnvironment &env);
    virtual ~MJPEGDeviceSource();
    virtual unsigned maxFrameSize() const;
    virtual void doGetNextFrame();

private:
    void deliverFrame();

private:
    JpegFrameParser parser;
    moodycamel::BlockingReaderWriterCircularBuffer<FrameData> frameDataQueue;
    static unsigned referenceCount;
    static EventTriggerId eventTriggerId;
};

#endif
